
# q-expansion of newform 73.2.a.c, downloaded from the LMFDB on 29 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 73
weight = 2
poly_data = [-3, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 1], [1, -1], [0, -1], [-1, 0], [3, 1], [-1, 1], [-3, 2], [-7, 0], [6, 1], [3, -4], [2, 2], [5, -2], [-6, 0], [5, -4], [9, 0], [-3, 4], [0, 0], [-4, -1], [5, -6], [3, -3], [1, 0], [-1, 3], [6, -5], [3, 6], [-1, -3], [-6, -2], [-1, 4], [-6, 3], [-1, 5], [9, -1], [-4, 1], [3, 3], [21, 0], [2, -1], [-6, -4], [14, -1], [-4, 3], [-1, -1], [12, -7], [-6, 3], [-3, -3], [17, -4], [-3, -10], [-16, 2], [9, 0], [-1, 11], [-13, 0], [-10, 0], [-6, 9], [5, -9], [3, -6], [6, 5], [5, -10], [-12, 3], [-3, 7], [15, -12], [-12, 3], [-16, 6], [-16, 7], [-9, -6], [-16, 8], [0, 4], [8, -4], [0, -9], [20, -5], [-12, 0], [-16, 4], [-4, 15], [-18, -6], [-1, 3], [6, -7], [9, 5], [-4, 2], [23, -6], [-4, 16], [-3, 2], [-9, 11], [-16, 6], [0, 2], [11, -4], [3, -5], [-13, 8], [12, 4], [-4, -11], [11, -14], [0, 6], [-15, -7], [8, -21], [27, 0], [-13, 18], [21, 6], [-18, 18], [20, 0], [24, -1], [-13, 12], [0, -9], [9, -18], [3, 10], [8, 2], [-7, -9], [17, -9], [9, 6], [-6, -10], [-18, 0], [14, 0], [-16, -6], [-9, 2], [-27, 3], [-6, -3], [26, -9], [11, -10], [11, 15], [-33, 0], [11, 10], [-1, -7], [-3, 6], [-16, 8], [0, 9], [-27, 5], [-3, 12], [29, 5], [5, 2], [-6, -16], [15, 12], [-7, -14], [-18, 2], [-25, 1], [6, -6], [-4, -19], [5, -6], [2, -9], [45, -6], [-4, -6], [8, -16], [6, 5], [29, -1], [27, 0], [20, -4], [-9, 3], [-24, 18], [-10, -4], [27, -8], [20, 5], [27, -3], [11, -12], [18, -3], [-28, 0], [-3, 9], [41, 0], [48, -3], [11, 8], [6, 3], [-22, 25], [21, 3], [-40, 8], [9, -6], [-16, 6], [12, 14], [-16, 6], [33, -9], [18, 7], [0, -14], [-4, -21], [-18, 12], [36, 5], [-3, 6], [29, -6], [-31, -1]]
